#!/usr/bin/env node
'use strict'

const fs = require('fs');
const path = require('path');
const moment = require('moment');
const program = require('commander');
const inquirer = require('inquirer');
const generator = require('./utils/generator')

let nameRule = '命名规则[dir-dir-pagename],例components-base-detail => components/base/detail.vue | pages-userInfo => pages/userInfo.vue';

program.option('-u, --use [template-name]', '使用模板创建新页面文件,模板名称如果不传将默认使用default')
// .option('-m, --mode', '不加-m默认使用.vue文件形式，加-m使用文件夹形式创建')
    .usage(`<page-name> -u <template-name>, ${nameRule}`)
    .parse(process.argv)

//检查是否存在autoTemplate目录
let isExistTemplatedir = fs.existsSync(path.resolve(`autoTemplate`));
let isExistDefault = fs.existsSync(path.resolve(`autoTemplate/default.vue`));
if (!isExistTemplatedir) {
    fs.mkdirSync(path.resolve(`autoTemplate`));
    createdDefaultTemplate();
} else if (!isExistDefault) {
    createdDefaultTemplate();
}

// 输入的页面名称
let inputPageName = program.args[0];
if (!inputPageName) {
    console.error('输入文件名称呀！');
    program.help();
}

// 输入的模板名称，默认template/default
let templateName = program.use || 'default';
//解析pagename为路径
let dirNames = inputPageName.replace(/-/g, '/');
let newPath = path.resolve(`src/pages/${dirNames.substring(0, dirNames.lastIndexOf('/'))}/`);
let pageName = inputPageName;

//判断是否已经存在此文件 && 路径是否存在
let isExistPath = fs.existsSync(path.resolve(`${newPath}`));
let isExistPage = fs.existsSync(path.resolve(`${newPath}/${pageName}.vue`));
if (isExistPage) {
    let mess = path.resolve(`src/pages/${dirNames}.vue`) + '已存在';
    throw new Error(mess);
} else {
    if (!isExistPath) {
        createdPathConfirm();
        // console.log(`#####请检查当前是否在项目根目录下或者检查项目名称是否有误#####${nameRule}`)
    } else {
        createdFile();
    }
}


//创建目录
function createdPath() {
    let dirList = inputPageName.split('-'), len = dirList.length;
    dirList.map((item, i) => {
        if (i < len - 1) {
            let arr = dirList.slice(0, i + 1);
            !fs.existsSync(path.resolve('src/pages/' + arr.join('/'))) ? fs.mkdirSync(path.resolve('src/pages/' + arr.join('/'))) : '';
        }
    })
    createdFile();
}
//当目录不存在时，询问是否生成
function createdPathConfirm() {
    let promptList = [{
        type: "confirm",
        message: "当前输入目录不存在，是否需要自动创建？",
        name: "newPath",
        // suffix: "后缀",
    }];
    inquirer.prompt(promptList)
        .then(answer => {
            answer.newPath ? createdPath() : console.log(`#####请检查当前是否在项目根目录下或者检查项目名称是否有误#####${nameRule}`);
        })

}

//生成默认模板文件
function createdDefaultTemplate() {
    let file = fs.readFileSync(`template/default.vue`, 'utf-8');
    fs.writeFileSync(path.resolve(`autoTemplate/default.vue`), file);
}

//生成文件
function createdFile() {
    fs.readFile(path.resolve(`autoTemplate/${templateName}.vue`), (err, file) => {
        if (err) {
            throw(err)
            program.help()
        }
        file = file.toString('utf8').replace('#{User}', process.env.USER)
            .replace('#{Date}', moment(new Date()).format('YYYY-MM-DD HH:mm:ss'));
        fs.writeFile(`${newPath}/${pageName}.vue`, file, err => {
            if (err) {
                console.log(err)
                program.help()
            }
            console.log('创建成功');
        });
    })
}
